#include <sys/types.h>
#include <sys/socket.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "poll.h"
#include "hash.h"
#include "location.h"
#include "store.h"

hashtable_t pollers;

void
poller_init(void)
{
	pollers = hash_make(64, &hash_hf_charp,
	    (int (*)(void *, void *))&strcmp);
}

int
add_poller(char *user, int sock)
{
	if (poller_exists(user) != -1) {
		printf("%s is already listening for updates\n", user);
		return 0;
	}

	hash_insert(pollers, user, hash_int_to_intp(sock));

	return 1;
}

int
poller_exists(char *user)
{
	int *sock;

	sock = hash_get(pollers, user);
	if (sock == NULL)
		return -1;

	return *sock;
}

void
poller_remove(char *user)
{
	int *sock;

	sock = hash_get(pollers, user);
	if (sock == NULL)
		return;
	free(sock);

	hash_remove(pollers, user);
}

void
poller_distribute(char *updater, location_t *loc)
{
	subscriber_cursor_t *cursor;
	char *subscriber;
	char locstr[128];
	char buffer[1024];
	size_t len;

	cursor = store_get_subscribers(updater);
	if (cursor == NULL)
		return;

	location_to_string(loc, locstr);
	len = snprintf(buffer, sizeof(buffer), "newlocation \"%s\" %s\n",
	               updater, locstr);
	while ((subscriber = store_get_subscribers_next(cursor))) {
		int sock;

		printf("Trying to send to: %s\n", subscriber);
		sock = poller_exists(subscriber);
		if (sock == -1) /* Not connected */
			continue;

		send(sock, buffer, len, 0);
	}
	store_get_subscribers_end(cursor);
}
