%{

#include <stdio.h>
#include <stdlib.h>

#include "protocol.h"
#include "location.h"

typedef void* yyscan_t;

%}

%pure-parser

%token UPDATE SEARCH USER SERVER FETCH POLL
%token SUBSCRIBE SUBSCRIBERS NEWUSER
%token <str>STRING
%token LOCATION
%token LBRACKET RBRACKET
%token <number>NUMBER

%parse-param { yyscan_t scanner };
%lex-param { yyscan_t scanner };
%parse-param { protocol_parser_t *pstate };
%lex-param { protocol_parser_t *pstate };

%union {
	double number;
	char *str;
	location_t *loc;
	command_t *cmd;
}

%type <loc> location
%type <cmd> command
%type <cmd> cmd_update

%start start

%%

start:
     	command { pstate->command = $1; }

location:
	LBRACKET NUMBER NUMBER RBRACKET {
		$$ = locaton_new($2, $3);
	}
	;

cmd_update:
	UPDATE USER STRING STRING location {
		command_t *c = command_new(CMD_UPDATE_USER);

		c->args.update.user = $3;
		c->args.update.pass = $4;
		c->args.update.loc = $5;
		$$ = c;
	}
	|
	UPDATE STRING STRING location {
		command_t *c = command_new(CMD_UPDATE);

		c->args.update.user = $2;
		c->args.update.pass = $3;
		c->args.update.loc = $4;
		$$ = c;
	}
	;

command:
	cmd_update
	|
	FETCH STRING {
		command_t *c = command_new(CMD_FETCH);
		c->args.user = $2;

		$$ = c;
	}
	|
	SUBSCRIBERS STRING {
		command_t *c = command_new(CMD_SUBSCRIBERS);
		c->args.user = $2;

		$$ = c;
	}
	|
	SUBSCRIBE STRING STRING {
		command_t *c = command_new(CMD_SUBSCRIBE);
		c->args.subscribe.user = $3;
		c->args.subscribe.subscriber = $2;

		$$ = c;
	}
	|
	POLL STRING {
		command_t *c = command_new(CMD_POLL);
		c->args.user = $2;

		$$ = c;
	}
	|
	NEWUSER {
		command_t *c = command_new(CMD_NEWUSER);

		$$ = c;
	}
	;

%%

int
yyerror(yyscan_t scanner, protocol_parser_t *pstate, char *msg)
{
	printf("Fucked up :( %s\n", msg);

	return 0;
}
