#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <assert.h>

#include "queue.h"
#include "alloc.h"

queue_t *
queue_new(void)
{
	queue_t *q;

	q = malloc(sizeof(*q));
	memset(q, '\0', sizeof(*q));
	pthread_mutex_init(&q->lock, NULL);
	pthread_cond_init(&q->cond, NULL);

	return q;
}

void
queue_add(queue_t *q, queue_func f, void *obj)
{
	queue_elem_t *e = malloc(sizeof(*e));

	e->obj = obj;
	e->func = f;
	e->next = NULL;

	pthread_mutex_lock(&q->lock);
	if (q->tail == NULL) {
		q->tail = q->head = e;
	} else {
		q->tail->next = e;
		q->tail = e;
	}
	q->length++;
	pthread_cond_signal(&q->cond);
	pthread_mutex_unlock(&q->lock);
}

queue_elem_t *
queue_get(queue_t *q)
{
	queue_elem_t *e;

	pthread_mutex_lock(&q->lock);
	while (q->length == 0)
		pthread_cond_wait(&q->cond, &q->lock);
	e = q->head;
	assert(e != NULL);

	if (q->head == q->tail) {
		q->head = q->tail = NULL;
	} else {
		q->head = e->next;
	}
	q->length--;

	pthread_mutex_unlock(&q->lock);

	return e;
}

void
queue_processer(queue_t *q, int *keep_running)
{
	while (*keep_running) {
		queue_elem_t *e;

		e = queue_get(q);
		assert(e);

		e->func(e->obj);
	}
}
